@@ -75,34 +75,43 @@ struct _GstDeviceMonitorClass {
   gpointer _gst_reserved[GST_PADDING];
 };
 
+GST_API
 GType     gst_device_monitor_get_type (void);
 
-GstDeviceMonitor * gst_device_monitor_new  (void);
+GST_API
+GstDeviceMonitor * gst_device_monitor_new  (void) G_GNUC_WARN_UNUSED_RESULT;
 
-GstBus *  gst_device_monitor_get_bus (GstDeviceMonitor * monitor);
+GST_API
+GstBus *  gst_device_monitor_get_bus (GstDeviceMonitor * monitor) G_GNUC_WARN_UNUSED_RESULT;
 
-GList *   gst_device_monitor_get_devices (GstDeviceMonitor * monitor);
+GST_API
+GList *   gst_device_monitor_get_devices (GstDeviceMonitor * monitor) G_GNUC_WARN_UNUSED_RESULT;
 
 
+GST_API
 gboolean  gst_device_monitor_start (GstDeviceMonitor * monitor);
 
+GST_API
 void      gst_device_monitor_stop  (GstDeviceMonitor * monitor);
 
 
+GST_API
 guint     gst_device_monitor_add_filter (GstDeviceMonitor * monitor,
                                          const gchar      * classes,
                                          GstCaps          * caps);
+GST_API
 gboolean  gst_device_monitor_remove_filter (GstDeviceMonitor * monitor,
                                             guint filter_id);
+GST_API
+gchar **  gst_device_monitor_get_providers (GstDeviceMonitor * monitor) G_GNUC_WARN_UNUSED_RESULT;
 
-gchar **  gst_device_monitor_get_providers (GstDeviceMonitor * monitor);
-
+GST_API
 void      gst_device_monitor_set_show_all_devices (GstDeviceMonitor * monitor, gboolean show_all);
+
+GST_API
 gboolean  gst_device_monitor_get_show_all_devices (GstDeviceMonitor * monitor);
 
-#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstDeviceMonitor, gst_object_unref)
-#endif
 
 G_END_DECLS
 
