@@ -1,7 +1,7 @@
 /* GStreamer byte reader
  *
  * Copyright (C) 2008 Sebastian Dröge <sebastian.droege@collabora.co.uk>.
- * Copyright (C) 2009 Tim-Philipp Müller <tim centricular net>
+ * Copyright (C) 2009,2014 Tim-Philipp Müller <tim centricular net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,12 +26,55 @@
 #define GST_BYTE_READER_DISABLE_INLINES
 #include "gstbytereader.h"
 
+#include "gst/glib-compat-private.h"
 #include <string.h>
 
 /**
  * SECTION:gstbytereader
+ * @title: GstByteReader
  * @short_description: Reads different integer, string and floating point
  *     types from a memory buffer
+ * @symbols:
+ * - gst_byte_reader_skip_unchecked
+ * - gst_byte_reader_get_uint8_unchecked
+ * - gst_byte_reader_peek_uint8_unchecked
+ * - gst_byte_reader_get_int8_unchecked
+ * - gst_byte_reader_peek_int8_unchecked
+ * - gst_byte_reader_get_uint16_le_unchecked
+ * - gst_byte_reader_get_uint16_be_unchecked
+ * - gst_byte_reader_peek_uint16_le_unchecked
+ * - gst_byte_reader_peek_uint16_be_unchecked
+ * - gst_byte_reader_get_int16_le_unchecked
+ * - gst_byte_reader_get_int16_be_unchecked
+ * - gst_byte_reader_peek_int16_le_unchecked
+ * - gst_byte_reader_peek_int16_be_unchecked
+ * - gst_byte_reader_get_uint24_le_unchecked
+ * - gst_byte_reader_get_uint24_be_unchecked
+ * - gst_byte_reader_peek_uint24_le_unchecked
+ * - gst_byte_reader_peek_uint24_be_unchecked
+ * - gst_byte_reader_get_int24_le_unchecked
+ * - gst_byte_reader_get_int24_be_unchecked
+ * - gst_byte_reader_peek_int24_le_unchecked
+ * - gst_byte_reader_peek_int24_be_unchecked
+ * - gst_byte_reader_get_uint32_le_unchecked
+ * - gst_byte_reader_get_uint32_be_unchecked
+ * - gst_byte_reader_peek_uint32_le_unchecked
+ * - gst_byte_reader_peek_uint32_be_unchecked
+ * - gst_byte_reader_get_int32_le_unchecked
+ * - gst_byte_reader_get_int32_be_unchecked
+ * - gst_byte_reader_peek_int32_le_unchecked
+ * - gst_byte_reader_peek_int32_be_unchecked
+ * - gst_byte_reader_get_float32_le_unchecked
+ * - gst_byte_reader_get_float32_be_unchecked
+ * - gst_byte_reader_get_float64_le_unchecked
+ * - gst_byte_reader_get_float64_be_unchecked
+ * - gst_byte_reader_peek_float32_le_unchecked
+ * - gst_byte_reader_peek_float32_be_unchecked
+ * - gst_byte_reader_peek_float64_le_unchecked
+ * - gst_byte_reader_peek_float64_be_unchecked
+ * - gst_byte_reader_peek_data_unchecked
+ * - gst_byte_reader_get_data_unchecked
+ * - gst_byte_reader_dup_data_unchecked
  *
  * #GstByteReader provides a byte reader that can read different integer and
  * floating point types from a memory buffer. It provides functions for reading
@@ -42,7 +85,7 @@
  */
 
 /**
- * gst_byte_reader_new:
+ * gst_byte_reader_new: (skip)
  * @data: (in) (transfer none) (array length=size): data from which the
  *     #GstByteReader should read
  * @size: Size of @data in bytes
@@ -56,7 +99,7 @@
 GstByteReader *
 gst_byte_reader_new (const guint8 * data, guint size)
 {
-  GstByteReader *ret = g_slice_new0 (GstByteReader);
+  GstByteReader *ret = g_new0 (GstByteReader, 1);
 
   ret->data = data;
   ret->size = size;
@@ -76,7 +119,7 @@ gst_byte_reader_free (GstByteReader * reader)
 {
   g_return_if_fail (reader != NULL);
 
-  g_slice_free (GstByteReader, reader);
+  g_free (reader);
 }
 
 /**
@@ -99,6 +142,56 @@ gst_byte_reader_init (GstByteReader * reader, const guint8 * data, guint size)
   reader->byte = 0;
 }
 
+/**
+ * gst_byte_reader_peek_sub_reader: (skip)
+ * @reader: an existing and initialized #GstByteReader instance
+ * @sub_reader: a #GstByteReader instance to initialize as sub-reader
+ * @size: size of @sub_reader in bytes
+ *
+ * Initializes a #GstByteReader sub-reader instance to contain @size bytes of
+ * data from the current position of @reader. This is useful to read chunked
+ * formats and make sure that one doesn't read beyond the size of the sub-chunk.
+ *
+ * Unlike gst_byte_reader_get_sub_reader(), this function does not modify the
+ * current position of @reader.
+ *
+ * Returns: FALSE on error or if @reader does not contain @size more bytes from
+ *     the current position, and otherwise TRUE
+ *
+ * Since: 1.6
+ */
+gboolean
+gst_byte_reader_peek_sub_reader (GstByteReader * reader,
+    GstByteReader * sub_reader, guint size)
+{
+  return _gst_byte_reader_peek_sub_reader_inline (reader, sub_reader, size);
+}
+
+/**
+ * gst_byte_reader_get_sub_reader: (skip)
+ * @reader: an existing and initialized #GstByteReader instance
+ * @sub_reader: a #GstByteReader instance to initialize as sub-reader
+ * @size: size of @sub_reader in bytes
+ *
+ * Initializes a #GstByteReader sub-reader instance to contain @size bytes of
+ * data from the current position of @reader. This is useful to read chunked
+ * formats and make sure that one doesn't read beyond the size of the sub-chunk.
+ *
+ * Unlike gst_byte_reader_peek_sub_reader(), this function also modifies the
+ * position of @reader and moves it forward by @size bytes.
+ *
+ * Returns: FALSE on error or if @reader does not contain @size more bytes from
+ *     the current position, and otherwise TRUE
+ *
+ * Since: 1.6
+ */
+gboolean
+gst_byte_reader_get_sub_reader (GstByteReader * reader,
+    GstByteReader * sub_reader, guint size)
+{
+  return _gst_byte_reader_get_sub_reader_inline (reader, sub_reader, size);
+}
+
 /**
  * gst_byte_reader_set_pos:
  * @reader: a #GstByteReader instance
@@ -711,7 +804,7 @@ GST_BYTE_READER_PEEK_GET(64,gdouble,float64_be)
 /**
  * gst_byte_reader_get_data:
  * @reader: a #GstByteReader instance
- * @size: Size in bytes
+ * @size: (in): Size in bytes
  * @val: (out) (transfer none) (array length=size): address of a
  *     #guint8 pointer variable in which to store the result
  *
@@ -719,7 +812,6 @@ GST_BYTE_READER_PEEK_GET(64,gdouble,float64_be)
  * position if at least @size bytes are left and
  * updates the current position.
  *
- *
  * Returns: %TRUE if successful, %FALSE otherwise.
  */
 gboolean
@@ -732,7 +824,7 @@ gst_byte_reader_get_data (GstByteReader * reader, guint size,
 /**
  * gst_byte_reader_peek_data:
  * @reader: a #GstByteReader instance
- * @size: Size in bytes
+ * @size: (in): Size in bytes
  * @val: (out) (transfer none) (array length=size): address of a
  *     #guint8 pointer variable in which to store the result
  *
@@ -740,7 +832,6 @@ gst_byte_reader_get_data (GstByteReader * reader, guint size,
  * position if at least @size bytes are left and
  * keeps the current position.
  *
- *
  * Returns: %TRUE if successful, %FALSE otherwise.
  */
 gboolean
@@ -753,7 +844,7 @@ gst_byte_reader_peek_data (const GstByteReader * reader, guint size,
 /**
  * gst_byte_reader_dup_data:
  * @reader: a #GstByteReader instance
- * @size: Size in bytes
+ * @size: (in): Size in bytes
  * @val: (out) (transfer full) (array length=size): address of a
  *     #guint8 pointer variable in which to store the result
  *
@@ -773,29 +864,81 @@ gst_byte_reader_dup_data (GstByteReader * reader, guint size, guint8 ** val)
 
 /* Special optimized scan for mask 0xffffff00 and pattern 0x00000100 */
 static inline gint
-_scan_for_start_code (const guint8 * data, guint offset, guint size)
+_scan_for_start_code (const guint8 * data, guint size)
 {
-  guint i = 0;
+  guint8 *pdata = (guint8 *) data;
+  guint8 *pend = (guint8 *) (data + size - 4);
 
-  while (i <= (size - 4)) {
-    if (data[i + 2] > 1) {
-      i += 3;
-    } else if (data[i + 1]) {
-      i += 2;
-    } else if (data[i] || data[i + 2] != 1) {
-      i++;
+  while (pdata <= pend) {
+    if (pdata[2] > 1) {
+      pdata += 3;
+    } else if (pdata[1]) {
+      pdata += 2;
+    } else if (pdata[0] || pdata[2] != 1) {
+      pdata++;
     } else {
-      break;
+      return (pdata - data);
     }
   }
 
-  if (i <= (size - 4))
-    return i + offset;
-
   /* nothing found */
   return -1;
 }
 
+static inline guint
+_masked_scan_uint32_peek (const GstByteReader * reader,
+    guint32 mask, guint32 pattern, guint offset, guint size, guint32 * value)
+{
+  const guint8 *data;
+  guint32 state;
+  guint i;
+
+  g_return_val_if_fail (size > 0, -1);
+  g_return_val_if_fail ((guint64) offset + size <= reader->size - reader->byte,
+      -1);
+
+  /* we can't find the pattern with less than 4 bytes */
+  if (G_UNLIKELY (size < 4))
+    return -1;
+
+  data = reader->data + reader->byte + offset;
+
+  /* Handle special case found in MPEG and H264 */
+  if ((pattern == 0x00000100) && (mask == 0xffffff00)) {
+    gint ret = _scan_for_start_code (data, size);
+
+    if (ret == -1)
+      return ret;
+
+    if (value != NULL)
+      *value = (1 << 8) | data[ret + 3];
+
+    return ret + offset;
+  }
+
+  /* set the state to something that does not match */
+  state = ~pattern;
+
+  /* now find data */
+  for (i = 0; i < size; i++) {
+    /* throw away one byte and move in the next byte */
+    state = ((state << 8) | data[i]);
+    if (G_UNLIKELY ((state & mask) == pattern)) {
+      /* we have a match but we need to have skipped at
+       * least 4 bytes to fill the state. */
+      if (G_LIKELY (i >= 3)) {
+        if (value)
+          *value = state;
+        return offset + i - 3;
+      }
+    }
+  }
+
+  /* nothing found */
+  return -1;
+}
+
+
 /**
  * gst_byte_reader_masked_scan_uint32:
  * @reader: a #GstByteReader
@@ -819,7 +962,7 @@ _scan_for_start_code (const guint8 * data, guint offset, guint size)
  * Returns: offset of the first match, or -1 if no match was found.
  *
  * Example:
- * <programlisting>
+ * |[
  * // Assume the reader contains 0x00 0x01 0x02 ... 0xfe 0xff
  *
  * gst_byte_reader_masked_scan_uint32 (reader, 0xffffffff, 0x00010203, 0, 256);
@@ -836,47 +979,45 @@ _scan_for_start_code (const guint8 * data, guint offset, guint size)
  * // -> returns 2
  * gst_byte_reader_masked_scan_uint32 (reader, 0xffff0000, 0x02030000, 0, 4);
  * // -> returns -1
- * </programlisting>
+ * ]|
  */
 guint
 gst_byte_reader_masked_scan_uint32 (const GstByteReader * reader, guint32 mask,
     guint32 pattern, guint offset, guint size)
 {
-  const guint8 *data;
-  guint32 state;
-  guint i;
+  return _masked_scan_uint32_peek (reader, mask, pattern, offset, size, NULL);
+}
 
-  g_return_val_if_fail (size > 0, -1);
-  g_return_val_if_fail ((guint64) offset + size <= reader->size - reader->byte,
-      -1);
-
-  /* we can't find the pattern with less than 4 bytes */
-  if (G_UNLIKELY (size < 4))
-    return -1;
-
-  data = reader->data + reader->byte + offset;
-
-  /* Handle special case found in MPEG and H264 */
-  if ((pattern == 0x00000100) && (mask == 0xffffff00))
-    return _scan_for_start_code (data, offset, size);
-
-  /* set the state to something that does not match */
-  state = ~pattern;
-
-  /* now find data */
-  for (i = 0; i < size; i++) {
-    /* throw away one byte and move in the next byte */
-    state = ((state << 8) | data[i]);
-    if (G_UNLIKELY ((state & mask) == pattern)) {
-      /* we have a match but we need to have skipped at
-       * least 4 bytes to fill the state. */
-      if (G_LIKELY (i >= 3))
-        return offset + i - 3;
-    }
-  }
-
-  /* nothing found */
-  return -1;
+/**
+ * gst_byte_reader_masked_scan_uint32_peek:
+ * @reader: a #GstByteReader
+ * @mask: mask to apply to data before matching against @pattern
+ * @pattern: pattern to match (after mask is applied)
+ * @offset: offset from which to start scanning, relative to the current
+ *     position
+ * @size: number of bytes to scan from offset
+ * @value: (out): pointer to uint32 to return matching data
+ *
+ * Scan for pattern @pattern with applied mask @mask in the byte reader data,
+ * starting from offset @offset relative to the current position.
+ *
+ * The bytes in @pattern and @mask are interpreted left-to-right, regardless
+ * of endianness.  All four bytes of the pattern must be present in the
+ * byte reader data for it to match, even if the first or last bytes are masked
+ * out.
+ *
+ * It is an error to call this function without making sure that there is
+ * enough data (offset+size bytes) in the byte reader.
+ *
+ * Returns: offset of the first match, or -1 if no match was found.
+ *
+ * Since: 1.6
+ */
+guint
+gst_byte_reader_masked_scan_uint32_peek (const GstByteReader * reader,
+    guint32 mask, guint32 pattern, guint offset, guint size, guint32 * value)
+{
+  return _masked_scan_uint32_peek (reader, mask, pattern, offset, size, value);
 }
 
 #define GST_BYTE_READER_SCAN_STRING(bits) \
@@ -986,7 +1127,7 @@ GST_BYTE_READER_SKIP_STRING (32);
  * gst_byte_reader_peek_string:
  * @reader: a #GstByteReader instance
  * @str: (out) (transfer none) (array zero-terminated=1): address of a
- *     #gchar pointer varieble in which to store the result
+ *     #gchar pointer variable in which to store the result
  *
  * Returns a constant pointer to the current data position if there is
  * a NUL-terminated string in the data (this could be just a NUL terminator).
@@ -1002,7 +1143,7 @@ GST_BYTE_READER_SKIP_STRING (32);
  * gst_byte_reader_peek_string_utf8:
  * @reader: a #GstByteReader instance
  * @str: (out) (transfer none) (array zero-terminated=1): address of a
- *     #gchar pointer varieble in which to store the result
+ *     #gchar pointer variable in which to store the result
  *
  * Returns a constant pointer to the current data position if there is
  * a NUL-terminated string in the data (this could be just a NUL terminator).
@@ -1035,7 +1176,7 @@ gst_byte_reader_peek_string_utf8 (const GstByteReader * reader,
  * gst_byte_reader_get_string_utf8:
  * @reader: a #GstByteReader instance
  * @str: (out) (transfer none) (array zero-terminated=1): address of a
- *     #gchar pointer varieble in which to store the result
+ *     #gchar pointer variable in which to store the result
  *
  * Returns a constant pointer to the current data position if there is
  * a NUL-terminated string in the data (this could be just a NUL terminator),
@@ -1082,7 +1223,7 @@ gst_byte_reader_dup_string_utf##bits (GstByteReader * reader, type ** str) \
     *str = NULL; \
     return FALSE; \
   } \
-  *str = g_memdup (reader->data + reader->byte, size); \
+  *str = g_memdup2 (reader->data + reader->byte, size); \
   reader->byte += size; \
   return TRUE; \
 }
@@ -1091,7 +1232,7 @@ gst_byte_reader_dup_string_utf##bits (GstByteReader * reader, type ** str) \
  * gst_byte_reader_dup_string_utf8:
  * @reader: a #GstByteReader instance
  * @str: (out) (transfer full) (array zero-terminated=1): address of a
- *     #gchar pointer varieble in which to store the result
+ *     #gchar pointer variable in which to store the result
  *
  * Free-function: g_free
  *
@@ -1111,7 +1252,7 @@ GST_BYTE_READER_DUP_STRING (8, gchar);
  * gst_byte_reader_dup_string_utf16:
  * @reader: a #GstByteReader instance
  * @str: (out) (transfer full) (array zero-terminated=1): address of a
- *     #guint16 pointer varieble in which to store the result
+ *     #guint16 pointer variable in which to store the result
  *
  * Free-function: g_free
  *
@@ -1137,7 +1278,7 @@ GST_BYTE_READER_DUP_STRING (16, guint16);
  * gst_byte_reader_dup_string_utf32:
  * @reader: a #GstByteReader instance
  * @str: (out) (transfer full) (array zero-terminated=1): address of a
- *     #guint32 pointer varieble in which to store the result
+ *     #guint32 pointer variable in which to store the result
  *
  * Free-function: g_free
  *
