@@ -569,14 +569,16 @@ gst_identity_src_event (GstBaseTransform * trans, GstEvent * event)
           &start, &stop_type, &stop);
 
       GST_OBJECT_LOCK (identity);
-      gst_segment_init (&identity->seek_segment, fmt);
-      if (!gst_segment_do_seek (&identity->seek_segment, rate, fmt,
-              flags, start_type, start, stop_type, stop, NULL)) {
-        GST_WARNING_OBJECT (identity, "Could not run seek %" GST_PTR_FORMAT,
-            event);
-        GST_OBJECT_UNLOCK (identity);
+      if (identity->single_segment) {
+        gst_segment_init (&identity->seek_segment, fmt);
+        if (!gst_segment_do_seek (&identity->seek_segment, rate, fmt,
+                flags, start_type, start, stop_type, stop, NULL)) {
+          GST_WARNING_OBJECT (identity, "Could not handle %" GST_PTR_FORMAT,
+              event);
+          GST_OBJECT_UNLOCK (identity);
 
-        return FALSE;
+          return FALSE;
+        }
       }
       GST_OBJECT_UNLOCK (identity);
 
@@ -711,7 +713,7 @@ gst_identity_update_last_message_for_buffer (GstIdentity * identity,
   g_free (identity->last_message);
   identity->last_message = g_strdup_printf ("%s   ******* (%s:%s) "
       "(%" G_GSIZE_FORMAT " bytes, dts: %s, pts: %s, duration: %s, offset: %"
-      G_GINT64_FORMAT ", " "offset_end: % " G_GINT64_FORMAT
+      G_GINT64_FORMAT ", " "offset_end: %" G_GINT64_FORMAT
       ", flags: %08x %s, meta: %s) %p", action,
       GST_DEBUG_PAD_NAME (GST_BASE_TRANSFORM_CAST (identity)->sinkpad), size,
       print_pretty_time (dts_str, sizeof (dts_str), GST_BUFFER_DTS (buf)),
@@ -911,18 +913,20 @@ gst_identity_set_property (GObject * object, guint prop_id,
       sync = g_value_get_boolean (value);
       GST_OBJECT_LOCK (identity);
       if (sync != identity->sync) {
+        GstClock *system_clock = gst_system_clock_obtain ();
         identity->sync = sync;
         if (sync) {
           GST_OBJECT_FLAG_SET (identity, GST_ELEMENT_FLAG_PROVIDE_CLOCK);
           clock_message =
               gst_message_new_clock_provide (GST_OBJECT_CAST (identity),
-              gst_system_clock_obtain (), TRUE);
+              system_clock, TRUE);
         } else {
           GST_OBJECT_FLAG_UNSET (identity, GST_ELEMENT_FLAG_PROVIDE_CLOCK);
           clock_message =
               gst_message_new_clock_lost (GST_OBJECT_CAST (identity),
-              gst_system_clock_obtain ());
+              system_clock);
         }
+        gst_object_unref (system_clock);
       }
       GST_OBJECT_UNLOCK (identity);
       if (clock_message)
